%define mpiimpl openmpi
%define mpidir %_libexecdir/%mpiimpl

%define scalar_type real
%define ldir %_libexecdir/petsc-%scalar_type

%define somver 0
%define sover %somver.1.9
%define oname oofem
Name: oofem-%scalar_type
Version: 2.0
Release: alt1.svn20091001
Summary: Object Oriented Finite Element Code
License: %gpl2plus
Group: Sciences/Mathematics
Url: http://www.oofem.org/
Packager: Eugeny A. Rostovtsev (REAL) <real at altlinux.org>

# http://www.oofem.org/svn/trunk
Source: %oname-%version.tar.gz
#Source1: http://www.oofem.org/download/ckit.tar.gz
#Source2: http://www.oofem.org/download/elixir.tar.gz
Source3: main.C

Requires: lib%name = %version-%release

BuildRequires(pre): rpm-build-licenses
BuildPreReq: python-devel chrpath
BuildPreReq: %mpiimpl-devel gcc-c++ libimlxx-devel libslepc-%scalar_type-devel
BuildPreReq: libX11-devel libXt-devel libICE-devel libXaw-devel libXmu-devel
BuildPreReq: libparmetis-devel libhdf5-mpi-devel
BuildPreReq: libneXtaw-devel libXext-devel doxygen graphviz latex2html

%description
OOFEM is free finite element code with object oriented architecture for
solving mechanical, transport and fluid mechanics problems that operates on
various platforms.

The aim of this project is to develop efficient and robust tool for FEM
computations as well as to provide modular and extensible environment for
future development.

%package -n lib%name
Summary: Shared libraries of Object Oriented Finite Element Code
Group: System/Libraries
Requires: libpetsc-%scalar_type

%description -n lib%name
OOFEM is free finite element code with object oriented architecture for
solving mechanical, transport and fluid mechanics problems that operates on
various platforms.

The aim of this project is to develop efficient and robust tool for FEM
computations as well as to provide modular and extensible environment for
future development.

This package contains shared libraries of OOFEM.

%package -n lib%name-devel
Summary: Shared libraries of Object Oriented Finite Element Code
Group: Development/C++
Requires: lib%name = %version-%release
Requires: libpetsc-%scalar_type-devel

%description -n lib%name-devel
OOFEM is free finite element code with object oriented architecture for
solving mechanical, transport and fluid mechanics problems that operates on
various platforms.

The aim of this project is to develop efficient and robust tool for FEM
computations as well as to provide modular and extensible environment for
future development.

This package contains development files of OOFEM.

%package -n %oname-doc
Summary: User's and developer's documentation for OOFEM
Group: Documentation
BuildArch: noarch

%description -n %oname-doc
OOFEM is free finite element code with object oriented architecture for
solving mechanical, transport and fluid mechanics problems that operates on
various platforms.

The aim of this project is to develop efficient and robust tool for FEM
computations as well as to provide modular and extensible environment for
future development.

This package contains user's and developer's documentation for OOFEM.

%package -n %oname-tests
Summary: Tests for OOFEM
Group: Sciences/Mathematics
BuildArch: noarch

%description -n %oname-tests
OOFEM is free finite element code with object oriented architecture for
solving mechanical, transport and fluid mechanics problems that operates on
various platforms.

The aim of this project is to develop efficient and robust tool for FEM
computations as well as to provide modular and extensible environment for
future development.

This package contains tests for OOFEM.

%package -n libckit
Summary: Shared library of C Programmer's Toolbox
Group: System/Libraries

%description -n libckit
This is a C-language toolkit (Ckit), which consists of a number of
useful packages. These are made into a single support library. The
core of the Ckit is a list processing package (formerly C-toolkit) by
Robert A Zimmermann. It helped to build programs faster and more
comfortably.

%package -n libckit-devel
Summary: Development files of C Programmer's Toolbox
Group: Development/C
Requires: libckit = %version-%release

%description -n libckit-devel
This is a C-language toolkit (Ckit), which consists of a number of
useful packages. These are made into a single support library. The
core of the Ckit is a list processing package (formerly C-toolkit) by
Robert A Zimmermann. It helped to build programs faster and more
comfortably.

This package contains development files of C Programmer's Toolbox.

%package -n libckit-devel-doc
Summary: Documentation for C Programmer's Toolbox
Group: Development/Documentation
BuildArch: noarch

%description -n libckit-devel-doc
This is a C-language toolkit (Ckit), which consists of a number of
useful packages. These are made into a single support library. The
core of the Ckit is a list processing package (formerly C-toolkit) by
Robert A Zimmermann. It helped to build programs faster and more
comfortably.

This package contains development documentation for C Programmer's
Toolbox.

%package -n libelixir
Summary: Extension LIbrary of X-based Interactive gRaphics
Group: System/Libraries
Requires: libckit = %version-%release

%description -n libelixir
ELIXIR is a set of utilities enabling the programmer to interactively
display and manipulate three-dimensional graphic entities within
multiple windows on a monitor screen.

%package -n libelixir-devel
Summary: Development files of Extension LIbrary of X-based Interactive gRaphics
Group: Development/C
Requires: libelixir = %version-%release
Requires: libckit-devel = %version-%release

%description -n libelixir-devel
ELIXIR is a set of utilities enabling the programmer to interactively
display and manipulate three-dimensional graphic entities within
multiple windows on a monitor screen.

This package contains development files of ELIXIR.

%package -n libelixir-devel-doc
Summary: Documentation for Extension LIbrary of X-based Interactive gRaphics
Group: Development/Documentation
BuildArch: noarch

%description -n libelixir-devel-doc
ELIXIR is a set of utilities enabling the programmer to interactively
display and manipulate three-dimensional graphic entities within
multiple windows on a monitor screen.

This package contains development documentation for ELIXIR.

%prep
%setup
sed -i 's|@SOMVER@|%somver|g' */src/Makefile.in base/main_makefile.in
sed -i 's|@SOVER@|%sover|g' */src/Makefile.in base/main_makefile.in
%if "%scalar_type" == "real"
pushd doc
tar -cf ../docs.tar $(find ./ -name '*.pdf')
popd
%endif
pushd src
tar -cf ../incs.tar $(find ./ -name '*.h')
popd

%build
mpi-selector --set %mpiimpl
source /usr/bin/petsc-%scalar_type.sh
export PETSC_INCLUDE=$PETSC_DIR/include
%add_optflags %optflags_shared -I$PETSC_INCLUDE
TOP=$PWD

# function for build main package

function buildIt() {
	%configure $3 $4 \
		--with-CKITDIR=$PWD/Ckit \
		--with-ELIXIRDIR=$PWD/Elixir \
		--with-MPIDIR=%mpidir \
		--with-x \
		--enable-iml \
		--with-IMLDIR=%_includedir \
		--enable-petsc \
		--enable-slepc \
		--with-PETSCDIR=$PETSC_DIR \
		--enable-parmetis \
		--with-PARMETISDIR=%prefix \
		--enable-dss \
		--enable-sm \
		--enable-tm \
		--enable-fm \
		OOFEM_TARGET=$1 \
		CPPFLAGS="-I$TOP/Ckit/include -I$TOP/Elixir/include" \
		LIBS="-L$TOP/Ckit/src -L$TOP/Elixir/src"
	if [ "$NO_BUILD" = "" ]; then
		pushd targets/default
		export LIBNAME=lib$2
		if [ "$1" = "default" ]; then
			%make_build
		else
			%make_build OOFEM_TARGET=$1
		fi 
		rm -f $(find ./ -name '*.o')
		if [ "$1" != "default" ]; then
			mv include/* ../$1/include/
		fi
		popd
	fi
}

# prepare

mkdir -p targets/default/include
mkdir -p targets/oofem-release/include
mkdir -p targets/poofem-release/include
mkdir objs
sed -i "s|(PWD)|$PWD|g" configure.in
%autoreconf

NO_BUILD=1
buildIt default poofem --enable-poofem
NO_BUILD=

# create necessary objects

function buildCommon() {
	mpicxx %optflags %optflags_shared -I../src/oofemlib \
		-I../src/oofemlib/xfem -I%ldir/include -I../src/sm \
		-I../src/tm -I../src/fm -I../Elixir/include \
		-I../Ckit/include -I../targets/default/include \
		-D__SM_MODULE -D__TM_MODULE -D__FM_MODULE \
		-D__USE_MPI -D__PETSC_MODULE $1 -c *.C
}

pushd objs
cp ../src/main/* ./
cp -f %SOURCE3 ./
rm -f maindebug.C
mkdir tmp

# parallel
buildCommon -D__PARALLEL_MODE
for i in $(ls *.o); do
	mv $i tmp/poofem-$i
done
# sequential
buildCommon
for i in $(ls *.o); do
	mv $i tmp/oofem-$i
done
# graphical
buildCommon -D__OOFEG
for i in $(ls *.o); do
	mv $i tmp/oofeg-$i
done

mv tmp/* ./
rmdir tmp
mv oofeg-oofeg.o oofeg.o

popd

# build necessary libraries

pushd Ckit/src
%autoreconf
%configure
%make_build
popd

pushd Elixir/src
%autoreconf
%configure \
	--with-x \
	CPPFLAGS="-I$TOP/Ckit/include" \
	LIBS="-L$TOP/Ckit/src"
%make_build
popd

# main build

buildIt poofem-release poofem --enable-poofem
buildIt oofem-release oofem
buildIt default oofeg --enable-oofeg --enable-oofeg-devel-interface

cp targets/oofem-release/bin/* targets/poofem-release/bin/* \
	targets/default/bin/

# need relink libesi & libelixir

pushd Elixir/src
rm -f libesi.*
%make_build OOFEG_LIB="-L../../targets/default -loofeg"
rm -f libelixir.*
%make_build OOFEG_LIB="-L../../targets/default -loofeg" ESI_LIB=-lesi
popd

%install

install -d %buildroot%ldir/bin
install -d %buildroot%_libdir
install -d %buildroot%ldir/lib
install -d %buildroot%ldir/include
install -d %buildroot%_includedir/Ckit
install -d %buildroot%_includedir/Elixir

install -m755 targets/default/bin/* %buildroot%ldir/bin
cp -P Ckit/src/*.so* Elixir/src/*.so* \
	%buildroot%_libdir/
cp -P targets/default/*.so* %buildroot%ldir/lib/
chmod -x %buildroot%ldir/lib/* %buildroot%_libdir/*.so*
install -p -m644 Ckit/include/*.h %buildroot%_includedir/Ckit
install -p -m644 Elixir/include/*.h %buildroot%_includedir/Elixir
pushd %buildroot%ldir/include
tar -xf $OLDPWD/incs.tar
popd

# tests & docs

%if "%scalar_type" == "real"
install -d %buildroot%_datadir/%oname
cp -fR tests %buildroot%_datadir/%oname/

install -d %buildroot%_docdir/%oname
install -d %buildroot%_docdir/Ckit
install -d %buildroot%_docdir/Elixir
install -p -m644 Ckit/docs/* %buildroot%_docdir/Ckit
install -p -m644 Elixir/doc/* %buildroot%_docdir/Elixir
pushd %buildroot%_docdir/%oname
tar -xf $OLDPWD/docs.tar
popd
%endif

cp Ckit/README README.Ckit
cp Elixir/README README.Elixir

# fix rpath

pushd %buildroot%_libdir
for i in elixir esi; do
	chrpath -r %ldir/lib:%mpidir/lib lib$i.so
done
popd

%files
%doc ChangeLog gpl.txt README
%ldir/bin/*

%files -n lib%name
%ldir/lib/*.so.*

%files -n lib%name-devel
%ldir/lib/*.so
%ldir/include/*

%if "%scalar_type" == "real"
%files -n %oname-doc
%_docdir/%oname

%files -n %oname-tests
%dir %_datadir/%oname
%_datadir/%oname/tests

%files -n libckit
%doc Ckit/README
%_libdir/libckit.so.*

%files -n libckit-devel
%_libdir/libckit.so
%_includedir/Ckit

%files -n libckit-devel-doc
%_docdir/Ckit

%files -n libelixir
%doc Elixir/README
%_libdir/libelixir.so.*
%_libdir/libesi.so.*

%files -n libelixir-devel
%_libdir/libelixir.so
%_libdir/libesi.so
%_includedir/Elixir

%files -n libelixir-devel-doc
%_docdir/Elixir
%endif

%changelog
* Mon Oct 5 2009 Eugeny A. Rostovtsev (REAL) <real at altlinux.org> 2.0-alt1.svn20091001
- Initial build for Sisyphus

